{  1991, Bowers Development Corp. }
{ UAMLibraryM.p }

{===========================================================}
{ Classes to support AppMaker user interface items: 		}
{															}
{ Class			(Subclass)		Description					}
{-----------------------------------------------------------}
{ TGrayLine		(TView)			Gray line					}
{ TPalette		(TCtlMgr)		Palette						}
{ TPictButton	(TCtlMgr)		Picture button				}
{ TPictCheckBox (TCtlMgr)		Picture check box			}
{ TPictRadio 	(TCtlMgr)		Picture radio button		}
{ TMultiPict 	(TCtlMgr)		Multi-picture control		}
{ TSlider 		(TCtlMgr)		Custom slider				}
{ TCustomCntl	(TCtlMgr)		Other custom control		}
{===========================================================}

unit UAMLibraryM;
interface

	uses
		SysEqu, Traps, ULoMem, 				{}
		UMacAppUtilities, UPatch, 	 			{}
		UObject, UViewCoords, UMemory, 		{}
		UFailure, UMenuSetup, UList, 			{}
		UMacApp, UTEView, UGridView,		{}
		UDialog, UAssociation; 					{}


	const
		kPaletteProcID = (200 * 16);			{ 3200 }

		kPictButtonProcID = (201 * 16);			{ 3216 }
		kPictCheckProcID = (201 * 16) + 1;		{ 3217 }
		kPictRadioProcID = (201 * 16) + 2;		{ 3218 }
		kMultiPictProcID = (201 * 16) + 3;		{ 3219 }

		kSliderProcID = (202 * 16);
		kSliderGrayContinuousID = (202 * 16);			{ 3232 }
		kSliderIndContinuousID = (202 * 16) + 1;		{ 3233 }
		kSliderGrayDiscreteID = (202 * 16) + 2;		{ 3234 }
		kSliderIndDiscreteID = (202 * 16) + 3;		{ 3235 }

		mMultiPictHit = 101;
		mPaletteHit = 102;
		mCustomHit = 103;
		mSliderHit = 104;

{----------}
	procedure ActionProcForTSlider (aCMgrControl: ControlHandle; partCode: integer);

{----------}
	type
		TGrayLine = object(TView)
				procedure Draw (area: Rect);
				Override;
			end; {TGrayLine}

	type
		PaletteTemplate = record
				itsPictureID: integer;
				itsNumberAcross: integer;
				itsNumberDown: integer;
				itsFrameSize: integer;
			end;
		PaletteTPtr = ^PaletteTemplate;

	type
		TPalette = object(TCtlMgr)
				procedure IPalette (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPictureID: integer; itsNumberAcross: integer; itsNumberDown: integer; itsFrameSize: integer);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TPalette}

	type
		PictControlTemplate = record
				itsPictureIDs: Str255;
			end;
		PictControlTPtr = ^PictControlTemplate;

	type
		TPictButton = object(TCtlMgr)
				procedure IPictButton (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TPictButton}

	type
		TPictCheckBox = object(TCtlMgr)
				procedure IPictCheckBox (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; isTurnedOn: Boolean);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure DoChoice (origView: TView; itsChoice: integer);
				Override;
				function IsOn: Boolean;
				procedure SetState (state: Boolean; redraw: Boolean);
				procedure Toggle (redraw: Boolean);
				procedure ToggleIf (matchState: Boolean; redraw: Boolean);
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TPictCheckBox}

	type
		TPictRadio = object(TCtlMgr)
				procedure IPictRadio (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; isTurnedOn: Boolean);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure DoChoice (origView: TView; itsChoice: integer);
				Override;
				function IsOn: Boolean;
				procedure SetState (state: Boolean; redraw: Boolean);
				procedure Toggle (redraw: Boolean);
				procedure ToggleIf (matchState: Boolean; redraw: Boolean);
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TPictRadio}

	type
		TMultiPict = object(TCtlMgr)
				procedure IMultiPict (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; curValue: integer);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure DoChoice (origView: TView; itsChoice: integer);
				Override;
				procedure Cycle (redraw: Boolean);
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TMultiPict}

	type
		SliderTemplate = record
				itsVariantCode: integer;
				itsPictureIDs: Str255;
			end;
		SliderTPtr = ^SliderTemplate;

	type
		TSlider = object(TCtlMgr)
				fDirection: VHSelect;	{ horizontal or vertical }

				procedure ISlider (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsDirection: VHSelect; itsPicts: Str255; itsProcID: integer; itsVal: Longint; itsMin: Longint; itsMax: Longint);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure DeltaValue (delta: VCoordinate);
				function DoMouseCommand (var theMouse: Point; var info: EventInfo; var hysteresis: Point): TCommand;
				Override;
				procedure TrackSlider (partCode: integer);
				procedure ActionProc (partCode: integer);
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TSlider}

	type
		CustomControlTemplate = record
				itsCntlID: integer;
			end;
		CustomControlTPtr = ^CustomControlTemplate;

	type
		TCustomCntl = object(TCtlMgr)
				fCntlID: integer;

				procedure ICustom (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsTitle: Str255; itsCntlID: integer);
				procedure IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
				Override;
				procedure WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
				Override;
				procedure Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
				Override;
			end; {TCustomCntl}

{----------}
implementation

	const
		kStdGrayLine = 'Line';
		kStdPalette = 'Palt';
		kStdPictButton = 'pBut';
		kStdPictCheckBox = 'pChk';
		kStdPictRadio = 'pRad';
		kStdMultiPict = 'pMul';
		kStdSlider = 'Sldr';
		kStdCustom = 'Cust';

{$S AMControlRes}
{----------}
	procedure ActionProcForTSlider (aCMgrControl: ControlHandle; partCode: integer);
		var
			aSlider: TSlider;
	begin
		aSlider := TSlider(GetCRefCon(aCMgrControl));
		FailNil(aSlider);

		aSlider.ActionProc(partCode);
	end; {ActionProcForTSlider}

{$S ARes}
{-------------------------------------------------------}
{ TGrayLine (TView)			Gray line					}
{-------------------------------------------------------}

{----------}
	procedure TGrayLine.Draw (area: Rect);
		var
			savePen: PenState;
	begin
		GetPenState(savePen);
		PenNormal;
		PenPat(gray);

		MoveTo(area.left, area.top);
		LineTo(area.right - 1, area.bottom - 1);

		SetPenState(savePen);
	end; {Draw}

{-------------------------------------------------------}
{ TPalette (TCtlMgr)		Palette						}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TPalette.IPalette (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPictureID: integer; itsNumberAcross: integer; itsNumberDown: integer; itsFrameSize: integer);
		var
			itsLabel: Str255;
	begin
		itsLabel[0] := chr(4);
		itsLabel[1] := chr(BSR(itsPictureID, 8));
		itsLabel[2] := chr(BAnd(itsPictureID, $FF));
		itsLabel[3] := chr(itsNumberAcross);
		itsLabel[4] := chr(itsNumberDown);
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsLabel, 1, 1, (itsNumberAcross * itsNumberDown), (kPaletteProcID + itsFrameSize));
		fDefChoice := mPaletteHit;
	end; {IPictButton}

{$S AInit}
{----------}
	procedure TPalette.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
			itsLabel: Str255;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mPaletteHit;
		ControlArea(itsArea);
		with PaletteTPtr(itsParams)^ do
			begin
				itsLabel[0] := chr(4);
				itsLabel[1] := chr(BSR(itsPictureID, 8));
				itsLabel[2] := chr(BAnd(itsPictureID, $FF));
				itsLabel[3] := chr(itsNumberAcross);
				itsLabel[4] := chr(itsNumberDown);
				CreateCMgrControl(itsArea, itsLabel, 1, 1, (itsNumberDown * itsNumberAcross), (kPaletteProcID + itsFrameSize));
			end; {with}

		OffsetPtr(itsParams, sizeof(PaletteTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TPalette.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			ptPtr: PaletteTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		ptPtr := PaletteTPtr(ExpandPtr(theResource, itsParams, sizeof(PaletteTemplate)));
		with ptPtr^ do
			begin
				itsPictureID := BOr(BSL(ord(theLabel[1]), 8), ord(theLabel[2]));
				itsNumberAcross := ord(theLabel[3]);
				itsNumberDown := ord(theLabel[4]);
				itsFrameSize := GetCVariant(fCMgrControl);
			end; {with}
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TPalette.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdPalette;
		gWResType := 'TPalette';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S AFields}
{----------}
	procedure TPalette.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TPalette', nil, bClass);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TPictButton (TCtlMgr)		Picture button				}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TPictButton.IPictButton (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255);
	begin
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsPicts, 0, 0, 1, kPictButtonProcID);
		fDefChoice := mButtonHit;
	end; {IPictButton}

{$S AInit}
{----------}
	procedure TPictButton.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mButtonHit;
		ControlArea(itsArea);
		with PictControlTPtr(itsParams)^ do
			begin
				CreateCMgrControl(itsArea, itsPictureIDs, 0, 0, 1, kPictButtonProcID);
			end; {with}

		OffsetPtrWStr(itsParams, sizeof(PictControlTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TPictButton.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			ptPtr: PictControlTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		ptPtr := PictControlTPtr(ExpandPtrWStr(theResource, itsParams, sizeof(PictControlTemplate), Length(theLabel)));
		CopyStr255(theLabel, PRStr(ptPtr^.itsPictureIDs));
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TPictButton.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdPictButton;
		gWResType := 'TPictButton';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S AFields}
{----------}
	procedure TPictButton.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TPictButton', nil, bClass);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TPictCheckBox (TCtlMgr)	Picture check box			}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TPictCheckBox.IPictCheckBox (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; isTurnedOn: Boolean);

	begin
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsPicts, 0, 0, 1, kPictCheckProcID);
		SetState(isTurnedOn, kDontRedraw);
		fDefChoice := mCheckBoxHit;
	end; {IPictCheckBox}

{$S AInit}
{----------}
	procedure TPictCheckBox.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mCheckBoxHit;
		ControlArea(itsArea);
		with PictControlTPtr(itsParams)^ do
			begin
				CreateCMgrControl(itsArea, itsPictureIDs, 0, 0, 1, kPictCheckProcID);
			end; {with}

		OffsetPtrWStr(itsParams, sizeof(PictControlTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TPictCheckBox.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			ptPtr: PictControlTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		ptPtr := PictControlTPtr(ExpandPtrWStr(theResource, itsParams, sizeof(PictControlTemplate), Length(theLabel)));
		CopyStr255(theLabel, PRStr(ptPtr^.itsPictureIDs));
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TPictCheckBox.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdPictCheckBox;
		gWResType := 'TPictCheckBox';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S ARes}
{----------}
	procedure TPictCheckBox.DoChoice (origView: TView; itsChoice: integer);
	begin
		if itsChoice = mCheckBoxHit then
			begin
				Toggle(kRedraw);
			end;
		inherited DoChoice(origView, itsChoice);
	end; {DoChoice}

{$S ARes}
{----------}
	function TPictCheckBox.IsOn: Boolean;
	begin
		IsOn := (GetLongVal <> 0);
	end; {IsOn}

{$S ARes}
{----------}
	procedure TPictCheckBox.SetState (state: Boolean; redraw: Boolean);
	begin
		SetLongVal(ord(state), redraw);
	end; {SetState}

{$S ARes}
{----------}
	procedure TPictCheckBox.Toggle (redraw: Boolean);
	begin
		SetLongVal(ord(not IsOn), redraw);
	end; {Toggle}

{$S ARes}
{----------}
	procedure TPictCheckBox.ToggleIf (matchState: Boolean; redraw: Boolean);
	begin
		if IsOn = matchState then
			begin
				SetLongVal(ord(not IsOn), redraw);
			end;
	end; {ToggleIf}

{$S AFields}
{----------}
	procedure TPictCheckBox.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TPictCheckBox', nil, bClass);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TPictRadio (TCtlMgr)		Picture radio button		}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TPictRadio.IPictRadio (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; isTurnedOn: Boolean);
	begin
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsPicts, 0, 0, 1, kPictRadioProcID);
		SetState(isTurnedOn, kDontRedraw);
		fDefChoice := mRadioHit;
	end; {IPictRadio}

{$S AInit}
{----------}
	procedure TPictRadio.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mRadioHit;
		ControlArea(itsArea);
		with PictControlTPtr(itsParams)^ do
			begin
				CreateCMgrControl(itsArea, itsPictureIDs, 0, 0, 1, kPictCheckProcID);
			end; {with}

		OffsetPtrWStr(itsParams, sizeof(PictControlTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TPictRadio.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			ptPtr: PictControlTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		ptPtr := PictControlTPtr(ExpandPtrWStr(theResource, itsParams, sizeof(PictControlTemplate), Length(theLabel)));
		CopyStr255(theLabel, PRStr(ptPtr^.itsPictureIDs));
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TPictRadio.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdPictRadio;
		gWResType := 'TPictRadio';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S ARes}
{----------}
	procedure TPictRadio.DoChoice (origView: TView; itsChoice: integer);
	begin
		if (itsChoice = mRadioHit) & (not IsOn) then
			begin
				Toggle(kRedraw);
			end;
		inherited DoChoice(origView, itsChoice);
	end; {DoChoice}

{$S ARes}
{----------}
	function TPictRadio.IsOn: Boolean;
	begin
		IsOn := (GetLongVal <> 0);
	end; {IsOn}

{$S ARes}
{----------}
	procedure TPictRadio.SetState (state: Boolean; redraw: Boolean);
	begin
		SetLongVal(ord(state), redraw);
	end; {SetState}

{$S ARes}
{----------}
	procedure TPictRadio.Toggle (redraw: Boolean);
	begin
		SetLongVal(ord(not IsOn), redraw);
	end; {Toggle}

{$S ARes}
{----------}
	procedure TPictRadio.ToggleIf (matchState: Boolean; redraw: Boolean);
	begin
		if IsOn = matchState then
			begin
				SetLongVal(ord(not IsOn), redraw);
			end;
	end; {ToggleIf}

{$S AFields}
{----------}
	procedure TPictRadio.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TPictRadio', nil, bClass);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TMultiPict (TCtlMgr)		Multi-picture control		}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TMultiPict.IMultiPict (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsPicts: Str255; curValue: integer);
		var
			numberOfPicts: integer;		{ number of possible states for the control }
	begin
		numberOfPicts := (length(itsPicts)) div 2;	{ 2 bytes for each picture ID }
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsPicts, curValue, 1, numberOfPicts, kMultiPictProcID);
		fDefChoice := mMultiPictHit;
	end; {IPictRadio}

{$S AInit}
{----------}
	procedure TMultiPict.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
			numberOfPicts: integer;		{ number of possible states for the control }
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mMultiPictHit;
		ControlArea(itsArea);
		with PictControlTPtr(itsParams)^ do
			begin
				numberOfPicts := (length(itsPictureIDs)) div 2;	{ 2 bytes per picture ID }
				CreateCMgrControl(itsArea, itsPictureIDs, 1, 1, numberOfPicts, kMultiPictProcID);
			end; {with}

		OffsetPtrWStr(itsParams, sizeof(PictControlTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TMultiPict.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			ptPtr: PictControlTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		ptPtr := PictControlTPtr(ExpandPtrWStr(theResource, itsParams, sizeof(PictControlTemplate), Length(theLabel)));
		CopyStr255(theLabel, PRStr(ptPtr^.itsPictureIDs));
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TMultiPict.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdMultiPict;
		gWResType := 'TMultiPict';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S ARes}
{----------}
	procedure TMultiPict.DoChoice (origView: TView; itsChoice: integer);
	begin
		if itsChoice = mMultiPictHit then
			begin
				Cycle(kRedraw);
			end;
		inherited DoChoice(origView, itsChoice);
	end; {DoChoice}

{$S ARes}
{----------}
	procedure TMultiPict.Cycle (redraw: Boolean);
		var
			curVal: integer;
	begin
		curVal := GetVal;

		if curVal = GetMax then
			begin
				curVal := GetMin;
			end
		else
			begin
				curVal := curVal + 1;
			end;
		SetVal(curVal, redraw);
	end; {Cycle}

{$S AFields}
{----------}
	procedure TMultiPict.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TMultiPict', nil, bClass);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TSlider (TCtlMgr)			Custom slider				}
{-------------------------------------------------------}

{$S AOpen}
{----------}
	procedure TSlider.ISlider (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsDirection: VHSelect; itsPicts: Str255; itsProcID: integer; itsVal: Longint; itsMin: Longint; itsMax: Longint);
	begin
		ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsPicts, itsVal, itsMin, itsMax, itsProcID);
		fDirection := itsDirection;
		fDefChoice := mSliderHit;
	end; {ISlider}

{$S AInit}
{----------}
	procedure TSlider.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mSliderHit;
		ControlArea(itsArea);
		with SliderTPtr(itsParams)^ do
			begin
				CreateCMgrControl(itsArea, itsPictureIDs, 0, 0, 10, kSliderProcID + itsVariantCode);
			end; {with}

		with itsArea do
			begin
				if (bottom - top) >= (right - left) then
					begin
						fDirection := v;
					end
				else
					begin
						fDirection := h;
					end;
			end; {with}

		OffsetPtrWStr(itsParams, sizeof(SliderTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TSlider.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			theLabel: Str255;
			stPtr: SliderTPtr;
	begin
		inherited WRes(theResource, itsParams);

		GetText(theLabel);
		stPtr := SliderTPtr(ExpandPtrWStr(theResource, itsParams, sizeof(SliderTemplate), Length(theLabel) + 2));
		stPtr^.itsVariantCode := 0;		{ should be control's variant code }
		CopyStr255(theLabel, PRStr(stPtr^.itsPictureIDs));
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TSlider.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdSlider;
		gWResType := 'TSlider';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S ARes}
{----------}
	procedure TSlider.DeltaValue (delta: VCoordinate);
	begin
		if (delta <> 0) then
			begin
				if (delta > 0) then
					begin
						delta := Min(delta, fLongMax - fLongVal);
					end
				else
					begin
						delta := Max(delta, fLongMin - fLongVal);
					end;

				SetLongVal(fLongVal + delta, true);
			end;
	end; {DeltaValue}

{$S ASelCommand}
{----------}
	function TSlider.DoMouseCommand (var theMouse: Point; var info: EventInfo; var hysteresis: Point): TCommand;
		var
			partCode: integer;
			oldLongValue: VCoordinate;
			newLongValue: VCoordinate;
	begin
	{$IFC qDebug}
		AssumeFocused;
	{$ENDC}

		oldLongValue := fLongVal;
		partCode := TestControl(fCMgrControl, theMouse);

		case partCode of
			inUpButton, inDownButton, inPageUp, inPageDown: 
				begin
					partCode := TrackControl(fCMgrControl, theMouse, @ActionProcForTSlider);
					if fLongVal <> oldLongValue then
						begin
							DoChoice(self, fDefChoice);
						end;
				end;
			inThumb: 
				begin
					if TrackControl(fCMgrControl, theMouse, nil) = inThumb then
						begin
							if GetVal = GetMax then
								begin
									newLongValue := fLongMax;
								end
							else
								begin
									newLongValue := BSL(Longint(GetVal), fBitsToShift);
								end;
							SetLongVal(newLongValue, kRedraw);
						end;

					if fLongVal <> oldLongValue then
						begin
							DoChoice(self, fDefChoice);
						end;
				end;
		end; {case}

		DoMouseCommand := nil;
	end; {DoMouseCommand}

{$S ARes}
{----------}
	procedure TSlider.TrackSlider (partCode: integer);
	begin
		DeltaValue(1);		{ only one part code }
	end; {TrackSlider}

{$S ASelCommand}
{----------}
{ this is based on TScrollBar.ActionProc.  I'm not completely sure it's right }
	procedure TSlider.ActionProc (partCode: integer);
	begin
		if (partCode <> 0) then
			begin
				TrackSlider(partCode);		{ bounds checking? }
			end;
		Update;
		if Focus then
			begin
			end;
	end; {ActionProc}

{$S AFields}
{----------}
	procedure TSlider.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TSlider', nil, bClass);
		DoToField('fDirection', @fDirection, bVHSelect);
		inherited Fields(DoToField);
	end; {Fields}

{-------------------------------------------------------}
{ TCustomCntl (TCtlMgr)		Other custom control		}
{-------------------------------------------------------}

	type
		CntlTemplate = record
				boundsRect: Rect;
				value: integer;
				visible: Boolean;
				filler1: Boolean;
				max: integer;
				min: integer;
				procID: integer;
				refCon: longint;
				title: Str255;
			end;
		CntlTPtr = ^CntlTemplate;
		CntlTHndl = ^CntlTPtr;

{$S AOpen}
{----------}
	procedure TCustomCntl.ICustom (itsSuperView: TView; itsLocation: VPoint; itsSize: VPoint; itsHSizeDet: SizeDeterminer; itsVSizeDet: SizeDeterminer; itsTitle: Str255; itsCntlID: integer);
		var
			itsCNTL: Handle;
	begin
		fCntlID := itsCntlID;
		itsCNTL := Get1Resource('CNTL', itsCntlID);
		LoadResource(itsCNTL);
		HLock(itsCNTL);
		with CntlTHndl(itsCNTL)^^ do
			begin
				;
				ICtlMgr(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet, itsTitle, value, min, max, procID);
			end; {with}
		ReleaseResource(itsCNTL);
		fDefChoice := mCustomHit;
	end; {ICustom}

{$S AInit}
{----------}
	procedure TCustomCntl.IRes (itsDocument: TDocument; itsSuperView: TView; var itsParams: Ptr);
		var
			itsArea: Rect;
			itsCNTL: Handle;
	begin
		inherited IRes(itsDocument, itsSuperView, itsParams);

		fDefChoice := mCustomHit;
		ControlArea(itsArea);
		with CustomControlTPtr(itsParams)^ do
			begin
				fCntlID := itsCntlID;
				itsCNTL := Get1Resource('CNTL', itsCntlID);
				LoadResource(itsCNTL);
				HLock(itsCNTL);
				with CntlTHndl(itsCNTL)^^ do
					begin
						;
						CreateCMgrControl(itsArea, title, value, min, max, procID);
					end; {with}
				ReleaseResource(itsCNTL);
			end; {with}

		OffsetPtr(itsParams, sizeof(CustomControlTemplate));
	end; {IRes}

{$S WriteRes}
{----------}
	procedure TCustomCntl.WRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
		var
			ctPtr: CustomControlTPtr;
	begin
		inherited WRes(theResource, itsParams);

		ctPtr := CustomControlTPtr(ExpandPtr(theResource, itsParams, sizeof(CustomControlTemplate)));
		ctPtr^.itsCntlID := fCntlID;
	end; {WRes}

{$S WriteRes}
{----------}
	procedure TCustomCntl.WriteRes (theResource: ViewRsrcHndl; var itsParams: Ptr);
	begin
		gWResSignature := kStdCustom;
		gWResType := 'TCustomCntl';
		WRes(theResource, itsParams);
	end; {WriteRes}

{$S AFields}
{----------}
	procedure TCustomCntl.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: integer));
	begin
		DoToField('TCustomCntl', nil, bClass);
		DoToField('fCntlID', @fCntlID, bInteger);
		inherited Fields(DoToField);
	end; {Fields}

end. {UAMLibraryM.p}